﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CodeChannelTester;
using System.Collections;

namespace Plugin_Channel_Manual
{
    public class Plugin_Channel_Manual : Channel
    {
        public override String  Name         { get { return "Manual Error Specification"; } }
        public override String  Description  { get { return "User can manually adjust burst traffic to induce custom errors."; } }
        public override Version Version      { get { return new Version(1,0); } }
        
        public override Boolean Configurable { get { return false; } }
        public override Boolean Configured   { get { return true; } }


        // Return the last error produced by manual intervention.
        public override SoftBurst LastError { get { return _LastError; } }
        private SoftBurst _LastError = null;

        // If set, the user will no longer be prompted to adjust the stream;
        // all future streams will be adjusted by the last used error.
        public Boolean UseForAll = false;


        public override void Reset()
        {
            UseForAll = false;
            _LastError = null;
        }

        public override SoftBurst CarryTransmission(SoftBurst transmission)
        {   
            // If we're using the last specified error for all future transmissions,
            if(UseForAll)
            {
                // Apply the last error to the transmission; 
                // Any part of the error that didn't fit can be ignored;
                // Any part of the tranmission that exceeds the size of the error can remain unchanged.
                // Still probably more useful with fixed-length block codes and datasources.
                var len = (int)Math.Min(LastError.BurstData.Length, transmission.BurstData.Length);
                for(int i = 0; i < len; i++)
                    transmission.BurstData[i] += LastError.BurstData[i];
                return transmission;
            }
            else
            {
                // Display the user interface for manual error adjustment.
                var trans_original = transmission.Copy();
                var dialog = new SoftBurstAdjust( transmission, this );
                dialog.ShowDialog();
                var trans_modified = transmission;

                // Determine and store the error produced.
                _LastError = SoftBurst.EditDifference(trans_original, trans_modified);

                return transmission;
            }
        }


    }
}
