﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using CodeChannelTester;

namespace Plugin_Channel_Manual
{

    // User interface for manually adjusting bits within a signal sample
    public partial class SoftBurstAdjust : Window
    {

        // A list of changes to the signal
        public List<BurstDeltaItem> BurstArray;

        public SoftBurst Burst;        
        public Plugin_Channel_Manual Plugin;

        public SoftBurstAdjust( SoftBurst burst, Plugin_Channel_Manual plugin )
        {
            Burst = burst;
            Plugin = plugin;
            InitializeComponent();

            // Create a new list of changes to the singal, and use it to
            // populate our data grid presented to the user
            BurstArray = BurstDeltaItem.ListFromBurst(Burst);
            dgBurst.ItemsSource = BurstArray;
        }

        
        // User clicked "OK"
        private void btnOK_Click(object sender, RoutedEventArgs e)
        {   
            // Apply each change that we've tracked to the signal.
            foreach( var bd in BurstArray )
                Burst.BurstData[bd.Index] = bd.Modified;

            this.Close();
        }

        // User clicked "Okay for All Bursts"
        private void btnOKAll_Click(object sender, RoutedEventArgs e)
        {
            // It can't be undone, and changes a lot, so confirm.
            var res = System.Windows.MessageBox.Show(this, 
                "Use the difference between input and output channel values for all future bursts?", 
                "Use for all?", 
                MessageBoxButton.OKCancel);
            if (!res.HasFlag( System.Windows.MessageBoxResult.OK )) 
                return;

            // Configure plugin to use the current change set)
            Plugin.UseForAll = true;
            // Apply changeset to current sample
            btnOK_Click( sender, e );
        }


        // Tracks the the original and modified values for each bit within a signal.
        // Presents only the Modified field as an adjustable field to the user-display
        // data grid.
        public class BurstDeltaItem
        {
            private int _index;
            public int Index { get { return _index; } }

            private double _original;
            public double Original { get { return _original; } }

            private double _modified;
            public double Modified { get { return _modified; } set { _modified = value; } }


            public BurstDeltaItem(int idx, double origVal )
            {
                _index = idx;
                _original = origVal;
                _modified = origVal;
            }


            // produce a full list for the signal changelog of a given signal.
            public static List<BurstDeltaItem> ListFromBurst( SoftBurst sb )
            {
                var l = new List<BurstDeltaItem>();
                for(int i = 0; i < sb.BurstData.Length; i++)
                    l.Add( new BurstDeltaItem(i, sb.BurstData[i]) );
                return l;
            }
        }

    }
}
