﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Plugin_Channel_SimpleNoise
{
    
    // Adjust configuration of Channel using simple additive/multiplicative noises
    public partial class Configure : Window
    {

        private Plugin_Channel_SimpleNoise Plugin;

        public Configure(Plugin_Channel_SimpleNoise plugin)
        {
            Plugin = plugin;
            InitializeComponent();

            // Set window componenets to display current values.
            txtRNGSeed.Text  = plugin.RNG_Seed.ToString();
            txtNoiseAmp.Text  = plugin.Noise_Amp.ToString();
            txtNoiseMean.Text = plugin.Noise_Mean.ToString();
            txtNoiseVar.Text  = plugin.Noise_Variance.ToString();
            rbNoiseBehave_Additive.IsChecked       = plugin.Noise_Behave == Noise_Behaviour.Additive;
            rbNoiseBehave_Multiplicative.IsChecked = plugin.Noise_Behave == Noise_Behaviour.Multiplicative;
            rbNoiseType_Uniform.IsChecked  = plugin.RNG_Type == RNG_Type.Uniform;
            rbNoiseType_Gaussian.IsChecked = plugin.RNG_Type == RNG_Type.Gaussian;
        }

        // Validate, Save, Close.
        private void btnOK_Click(object sender, RoutedEventArgs e)
        {
            var succ = false;

            // Validate RNG seed
            int rng_Seed;
            succ = Int32.TryParse(txtRNGSeed.Text, out rng_Seed);
            if(!String.IsNullOrWhiteSpace(txtRNGSeed.Text) && (!succ || rng_Seed < 0))
            {
                System.Windows.MessageBox.Show( "RNG seed should be a positive integer or empty" );
                return;
            }
            
            // Validate noise amplitude
            double noise_Amp;
            succ = Double.TryParse(txtNoiseAmp.Text, out noise_Amp);
            if( !succ )
            {
                System.Windows.MessageBox.Show( "Noise amplitude should be a real value" );
                return;
            }

            // Validate noise mean
            double noise_Mean;
            succ = Double.TryParse(txtNoiseMean.Text, out noise_Mean);
            if( !succ )
            {
                System.Windows.MessageBox.Show( "Noise distribution mean should be a real value" );
                return;
            }

            // Validate noise variance
            double noise_Var;
            succ = Double.TryParse(txtNoiseVar.Text, out noise_Var);
            if( !succ )
            {
                System.Windows.MessageBox.Show( "Noise distribution variance should be a real value" );
                return;
            }

            // Save configuration

            Plugin.RNG_Seed = null;
            if(!String.IsNullOrWhiteSpace(txtRNGSeed.Text)) Plugin.RNG_Seed = rng_Seed;

            Plugin.Noise_Amp = noise_Amp;
            Plugin.Noise_Mean = noise_Mean;
            Plugin.Noise_Variance = noise_Var;

            if(rbNoiseType_Gaussian.IsChecked ?? false)
                Plugin.RNG_Type = RNG_Type.Gaussian;
            if(rbNoiseType_Uniform.IsChecked ?? false)
                Plugin.RNG_Type = RNG_Type.Uniform;
            if(rbNoiseBehave_Additive.IsChecked ?? false)
                Plugin.Noise_Behave = Noise_Behaviour.Additive;
            if(rbNoiseBehave_Multiplicative.IsChecked ?? false)
                Plugin.Noise_Behave = Noise_Behaviour.Multiplicative;

            this.Close();
        }



    }
}
