﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Plugin_Channel_SimpleNoise
{

    // A simle interface for random number generators
    public interface RNG
    {
        void Reset( int Seed1, int Seed2 );
        double GetNext();
    }

    // The type of random number generator/noise to produce
    public enum RNG_Type
    {
        Uniform,    // Uniform  distribution
        Gaussian    // Gaussian distribution / White noise
    }


    // Better uniform random number generator
    //<http://www.codeproject.com/Articles/25172/Simple-Random-Number-Generation>
    // Marsaglia, George. "Random number generation." (2003): 1499-1503.
    public class Uniform : RNG
    {
        private uint v1 = 1;
        private uint v2 = 1;

        public void Reset( int Seed1, int Seed2)
        {
            v1 = (uint)Seed1;
            v2 = (uint)Seed2;
        }
        public uint GetNextInt()
        {
            v1 = 36969 * (v1 & 65535) + (v1 >> 16);
            v2 = 18000 * (v2 & 65535) + (v2 >> 16);
            return (v1 << 16) + v2;
        }
        public double GetNext()
        {
            uint i = GetNextInt();
            return (i + 1.0) * 2.328306435454494E-10;
        }
    }

    // Generate Gaussian-distributed random numbers using Central-Limit theorem
    //<http://dspguru.com/dsp/howtos/how-to-generate-white-gaussian-noise>
    public class Gaussian : RNG
    {
        private Uniform UniformRNG;
        private const int N = 3;

        public Gaussian()
        {
            UniformRNG = new Uniform();
        }
        public void Reset( int Seed1, int Seed2)
        {
            UniformRNG.Reset( Seed1, Seed2 );
        }
        public double GetNext()
        {
            double cur = 0.0;
            for(int i = 0; i < N; i++)
                cur += UniformRNG.GetNext();
            cur /= N;
            return cur;
        }

    }


}
