﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CodeChannelTester;
using System.Collections;

namespace Plugin_Code_Repetition
{
    public class Plugin_Code_Repetition : Code
    {

        public override String  Name        { get { return "Repetition"; } }
        public override String  Description { get { return "Data is sent in triplicate (or some n times in parallel)."; } }
        public override Version Version     { get { return new Version(1,0); } }

        public override Boolean Configurable { get { return true; } }
        public override Boolean Configured   { get { return true; } }
        
        public override void Configure()
        {
            var dlg = new Configure(this);
            dlg.ShowDialog();
        }

        // Number of times that a signal sample should be repeated in the encoding.
        public int RepeatTimes = 3;
        // Soft decode uses a mean of the signal samples instead of a mode average.
        public bool SoftDecode = false;

        public override SoftBurst EncodeBurst(System.Collections.BitArray inStream)
        {
            var ba = new BitArray(inStream.Count * RepeatTimes);
            // Repeat the signal sample n times. (n times repetition in parallel)
            for(int i = 0; i < inStream.Count; i++)
            for(int j = 0; j < RepeatTimes; j++)
                ba[(i*RepeatTimes)+j] = inStream[i];

            return SoftBurst.PolarEncode( ba );
        }
        public override BitArray DecodeBurst(SoftBurst inStream)
        {
            if(inStream.BurstData.Length % RepeatTimes != 0)
                throw new Exception("Something strange happened... data size mismatch in non-insertion/deletion channel.");

            var decoded = new BitArray( inStream.BurstData.Length / RepeatTimes );

            if(SoftDecode)
            {
                // Use mean average.
                for(int i = 0; i < decoded.Count; i++)
                {
                    double sum = 0.0;
                    for(int j = 0; j < RepeatTimes; j++)
                        sum += inStream.BurstData[(i*RepeatTimes)+j];
                    decoded[i] = sum >= 0.0;
                }
            }
            else
            {
                // Use mode average.
                var quantized = SoftBurst.QuantizeDecode(inStream);
                for(int i = 0; i < decoded.Count; i++)
                {
                    int count_0 = 0;
                    int count_1 = 0;
                    for(int j = 0; j < RepeatTimes; j++)
                    {
                        if(quantized[(i*RepeatTimes)+j])
                                count_1++;
                        else    count_0++;
                    }
                    decoded[i] = count_1 >= count_0;
                }
            }

            return decoded;
        }

    }
}
