﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CodeChannelTester;
using System.Collections;

namespace Plugin_DataSource_Exhaustive
{
    public class Plugin_DataSource_Exhaustive : DataSource
    {

        public override String  Name        { get { return "All Values of Length n"; } }
        public override String  Description { get { return "Data stream produced from all possible bit arrays withing a length range."; } }
        public override Version Version     { get { return new Version(1,0); } }

        public override Boolean Configurable { get { return true; } }
        public override Boolean Configured   { get { return true; } }


        public int Alignment     = 2;
        public int LengthMin     = 6;
        public int LengthMax     = 8;

        public override void Configure()
        {
            var dlg = new Configure(this);
            dlg.ShowDialog();
        }

        public override void Reset()
        {
            LastBurst = null;
        }

        private BitArray LastBurst = null;
        public override BitArray GetNextBurst()
        {
            if(LastBurst == null)
            {
                LastBurst = new BitArray(LengthMin);
                return LastBurst; // return immediately; don't add.
            }

            // Adder. Add 1 to each bigit, carrying the extra if needed.
            bool carry = true;
            for(int i = LastBurst.Count-1; i >= 0; i--)
            {
                var oldBit = LastBurst[i];
                LastBurst[i] = LastBurst[i] ^ carry; //XOR
                carry = oldBit & carry;
            }

            if(carry) // we still have a carry bit? we'd exceed the bitarray size.
            {
                var newSize = LastBurst.Count + Alignment;
                if(newSize <= LengthMax)
                        LastBurst = new BitArray(newSize);  // next size.
                else    LastBurst = null;                   // exhausted.
            }
            
            if (LastBurst==null) return null;
            return (BitArray)LastBurst.Clone();
        }


    }
}
