﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace Plugin_DataSource_Random
{
    
    // Adjust configuration of DataSource using randomly generated bytes
    public partial class Configure : Window
    {

        private Plugin_DataSource_Random Plugin_DataSource_Random;

        public Configure( Plugin_DataSource_Random pluginDS )
        {
            Plugin_DataSource_Random = pluginDS;
            InitializeComponent();

            // Set window componenets to display current values.
            var plugin = Plugin_DataSource_Random;
            txtSamples.Text   = plugin.BurstCount.ToString();
            txtAlignment.Text = plugin.Alignment.ToString();
            txtLengthAvg.Text = plugin.AverageSize.ToString();
            txtLengthDev.Text = plugin.SizeDeviation.ToString();
            txtRNGSeed.Text   = plugin.RNG_Seed.ToString();
            
        }

        // Validate, Save, Close.
        private void btnOK_Click(object sender, RoutedEventArgs e)
        {
            var succ = false;

            // Validate sample count
            int samples;
            succ = Int32.TryParse(txtSamples.Text, out samples);
            if( !succ || samples <= 0 )
            {
                System.Windows.MessageBox.Show( "Sample count should be a positive integer > 0" );
                return;
            }

            // Validate sample length alignment/divisibility
            int alignment;
            succ = Int32.TryParse(txtAlignment.Text, out alignment);
            if( !succ || alignment <= 0 )
            {
                System.Windows.MessageBox.Show( "Alignment should be a positive integer > 0" );
                return;
            }

            // Validate average sample length
            int lengthAvg;
            succ = Int32.TryParse(txtLengthAvg.Text, out lengthAvg);
            if( !succ || lengthAvg <= 0 )
            {
                System.Windows.MessageBox.Show( "Average length should be a positive integer > 0" );
                return;
            }

            // Validate sample length range
            int lengthDev;
            succ = Int32.TryParse(txtLengthDev.Text, out lengthDev);
            if( !succ || lengthDev < 0 )
            {
                System.Windows.MessageBox.Show( "Length deviation should be a positive integer" );
                return;
            }

            // Validate RNG seed
            int rng_seed;
            succ = Int32.TryParse(txtRNGSeed.Text, out rng_seed);
            if(!String.IsNullOrWhiteSpace(txtRNGSeed.Text) && (!succ || rng_seed < 0 ))
            {
                System.Windows.MessageBox.Show( "RNG seed should be a positive integer or empty" );
                return;
            }

            // Save configuration
            var plugin = Plugin_DataSource_Random;
            plugin.BurstCount = samples;
            plugin.Alignment = alignment;
            plugin.AverageSize = lengthAvg;
            plugin.SizeDeviation = lengthDev;
            plugin.RNG_Seed = null;
            if(!String.IsNullOrWhiteSpace(txtRNGSeed.Text)) plugin.RNG_Seed = rng_seed;

            this.Close();
        }


    }
}
