﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CodeChannelTester;
using System.Collections;

namespace Plugin_DataSource_Random
{
    public class Plugin_DataSource_Random : DataSource
    {

        public override String  Name        { get { return "Random Bytes"; } }
        public override String  Description { get { return "Data stream produced from default system RNG bytes of lengths specified."; } }
        public override Version Version     { get { return new Version(1,0); } }

        public override Boolean Configurable { get { return true; } }
        public override Boolean Configured   { get { return true; } }

        
        public int Alignment     = 8;
        public int AverageSize   = 128;
        public int SizeDeviation = 64;
        public int BurstCount    = 1024;
        public Int32? RNG_Seed   = null;
        public Random RNG        = new Random();

        public override void Configure()
        {
            var dlg = new Configure(this);
            dlg.ShowDialog();
        }

        public override void Reset()
        {
            currentBurst = 0;
            if(RNG_Seed.HasValue)
                RNG = new Random(RNG_Seed.Value);
        }

        private int currentBurst = 0;
        public override BitArray GetNextBurst()
        {
            if( ++currentBurst > BurstCount )
                return null; // Stop.

            int size = (int)(AverageSize + ( ((RNG.NextDouble()*2)-1.0) * SizeDeviation ));
            size += size % Alignment;
            size = (int)Math.Max(1, size);
            var ret = new BitArray(size);
            for(int i = 0; i < size; i++)
                ret[i] = RNG.NextDouble() >= 0.5;
            return ret;
        }

    }
}
