﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.IO;

namespace Plugin_DataSource_SamplesFromFile
{
    
    // Adjust configuration of DataSource using samples from files
    public partial class Configure : Window
    {

        private Plugin_DataSource_SamplesFromFile Plugin;

        public Configure( Plugin_DataSource_SamplesFromFile plugin )
        {
            Plugin = plugin;
            InitializeComponent();

            // Set window componenets to display current values.
            txtFileName.Text = Plugin.Filename;
            txtSamples.Text = Plugin.SampleCount.ToString();
            txtRNGSeed.Text = Plugin.RNG_Seed.ToString();
            rbSampling_Random.IsChecked =  Plugin.UseRandomSampling;
            rbSampling_Once.IsChecked   = !Plugin.UseRandomSampling;
        }

        // Validate, Save, Close.
        private void btnOK_Click(object sender, RoutedEventArgs e)
        {
            var succ = false;

            // Validate Sample Size
            int samples;
            succ = Int32.TryParse(txtSamples.Text, out samples);
            if( !succ || samples <= 0 )
            {
                System.Windows.MessageBox.Show( "Sample count should be a positive integer > 0" );
                return;
            }

            // Validate filename
            string filename = txtFileName.Text;
            if(String.IsNullOrWhiteSpace(filename))
            {
                System.Windows.MessageBox.Show( "A file must be specified from which to load strings to transmit." );
                return;
            }

            // Validate RNG seed
            int rng_seed;
            succ = Int32.TryParse(txtRNGSeed.Text, out rng_seed);
            if(!String.IsNullOrWhiteSpace(txtRNGSeed.Text) && (!succ || rng_seed < 0 ))
            {
                System.Windows.MessageBox.Show( "RNG seed should be a positive integer or empty" );
                return;
            }

            // Save configuration
            Plugin.Filename = filename;
            Plugin.UseRandomSampling = rbSampling_Random.IsChecked ?? false;
            Plugin.SampleCount = samples;
            Plugin.RNG_Seed = null;
            if(!String.IsNullOrWhiteSpace(txtRNGSeed.Text)) Plugin.RNG_Seed = rng_seed;
            
            this.Close();
        }


        // Prompt user for file use
        private void btnFileName_Click(object sender, RoutedEventArgs e)
        {
            try {
                var dialog = new Microsoft.Win32.OpenFileDialog();
                dialog.FileName = "input1";
                dialog.DefaultExt = ".txt";
                dialog.Filter = "Newline separated transmissions (*.*)|*.*";
                var dialog_result = dialog.ShowDialog(this);

                if(!(dialog_result??false)) return;
                if(!File.Exists(dialog.FileName)) return;

                txtFileName.Text = dialog.FileName;
            }
            catch (Exception ex) {
                System.Windows.MessageBox.Show( ex.ToString() );
            }
        }



    }
}
