﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CodeChannelTester;
using System.Collections;
using System.Data;
using System.IO;

namespace Plugin_DataSource_SamplesFromFile
{
    public class Plugin_DataSource_SamplesFromFile : DataSource
    {

        public override String  Name        { get { return "Samples from File"; } }
        public override String  Description { get { return "Data stream produced from entries within a formatted file"; } }
        public override Version Version     { get { return new Version(1,0); } }

        public override Boolean Configurable { get { return true; } }
        public override Boolean Configured   { get { return true; } }


        public String Filename;
        public string[] FileLines;
        public int[] SampleOrdering;
        public bool UseRandomSampling = true;
        public int SampleCount    = 1024;
        public int CurrentSample  = 0;
        public Int32? RNG_Seed   = null;
        public Random RNG        = new Random();

        public override void Configure()
        {
            var dlg = new Configure(this);
            dlg.ShowDialog();
        }

        public override void Setup()
        {
            FileLines = File.ReadAllLines( Filename, Encoding.ASCII );
        }

        public override void Reset()
        {
            CurrentSample = 0;
            if(RNG_Seed.HasValue)
                RNG = new Random(RNG_Seed.Value);

            // Prepare to shuffle ordering for fairness.
            SampleOrdering = new int[FileLines.Length];
            for(int i = 0; i < FileLines.Length; i++)
                SampleOrdering[i] = i;
            for(int i = 0; i < FileLines.Length; i++)
            {
                int idx = RNG.Next(FileLines.Length);
                int temp = SampleOrdering[idx];
                SampleOrdering[idx] = SampleOrdering[i];
                SampleOrdering[i] = temp;
            }
        }


        public override BitArray GetNextBurst()
        {
            CurrentSample++;

            if(UseRandomSampling && CurrentSample > SampleCount) return null;
            if(!UseRandomSampling && CurrentSample > FileLines.Length ) return null;

            int lineIdx;
            if(!UseRandomSampling)
                    lineIdx = CurrentSample-1;
            else    lineIdx = RNG.Next(FileLines.Length);
            
            String str = FileLines[lineIdx];
            byte[] bytes = System.Text.Encoding.ASCII.GetBytes( str );

            var ba = new BitArray( bytes );
            return ba;
        }


    }
}
