﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Plugin_SDK;
using System.Collections;


namespace CodeChannelTester
{
    /// <summary>
    /// Interface for Channel plugin objects.
    /// A Channel is responsible for inducing an amount of error to the passed signal.
    /// </summary>
    abstract public class Channel : Plugin_Configurable
    {

        // Provides the signal after having possibly applied an amount of error.
        // Simulates a tranmission channel model.
        // To remain within the limitations of the system, the input and output signals
        // should be of equal length.
        public abstract SoftBurst CarryTransmission( SoftBurst transmission );

        // An (optional) copy of the last error applied to a transmission.
        // If this is null, the the system will - for record keeping purposes - determine the 
        // error from the difference between values before and after being sent through the channel.
        public virtual SoftBurst LastError{ get { return null; } }

    }
}
