﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Plugin_SDK;
using System.Collections;


namespace CodeChannelTester
{
    /// <summary>
    /// Interface for Code plugin objects.
    /// A Code is responsible for encoding and decoding a signal with some, preferably 
    /// error-correcting, coding.
    /// The encoder and decoder methods should use common members for configuration, 
    /// but should not rely on any common state. Encoder and Decoder state should be 
    /// tracked separately.
    /// </summary>
    abstract public class Code : Plugin_Configurable
    {

        // Produces a encoded signal from a provided, BitArray signal.
        // It is not required that input and output lengths match.
        // The input represents a complete transmission burst; if a code would usually require 
        // additional signal samples to complete a block, the encoder should pad the end of 
        // the input signal with 0.
        // It is recommended that the encoded values be in line with polar-encoded values,
        // which can be accomplished through the SoftBurst helper functions.
        public abstract SoftBurst EncodeBurst( BitArray inStream );

        // Decodes an encoded signal back to the original BitArray signal.
        // It is not required that input and output lengths match.
        public abstract BitArray DecodeBurst( SoftBurst inStream );

    }
}
