﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Plugin_SDK
{
    /// <summary>
    /// Expected interface to be implemented by all plugin types which may require user configuration.
    /// The DataSource, Code, and Channel classes all inherit from Plugin_Configurable.
    /// All custom plugins made must implement the Configurable and Configured properties, and should 
    /// implement the remaining properties as needed.
    /// </summary>
    public abstract class Plugin_Configurable : Plugin
    {

        // indicates wether or not user configuration is possible
        public abstract Boolean Configurable{ get; }
        // indicates wether or not the test system may proceed with the current configuration
        public abstract Boolean Configured  { get; }


        // Present users with configuration options
        // This is called when the plugin is selected for a Datasource/Code/Channel, and the
        // user clicks the button to configure properties of the plugin. 
        // This should only return when the user has completed configuration.
        public virtual void Configure(){}

        // Perform intialization based on user configuration.
        // This will be called once, after the user requests testing to begin.
        // This should intiialize any state which is static or dependant on user configuration,
        // but which may persist between the user requesting testing to begin, so long as 
        // confguration has not changed. If the plugin was unselected, or reconfigured, this 
        // procedure will be called again the next time the users requests testing begins.
        public virtual void Setup(){}

        // Reset any temporary state to intialized values
        // Called every time the user requests testing to begin.
        public virtual void Reset(){}

    }
}
